@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #--
-# Copyright (C) 2009-2013 Thomas Leitner <t_leitner@gmx.at>
+# Copyright (C) 2009-2015 Thomas Leitner <t_leitner@gmx.at>
 #
 # This file is part of kramdown which is licensed under the MIT.
 #++
@@ -9,6 +9,7 @@
 
 require 'erb'
 require 'kramdown/utils'
+require 'kramdown/document'
 
 module Kramdown
 
@@ -59,26 +60,52 @@ module Kramdown
       end
       private_class_method(:new, :allocate)
 
+      # Returns whether the template should be applied before the conversion of the tree.
+      #
+      # Defaults to false.
+      def apply_template_before?
+        false
+      end
+
+      # Returns whether the template should be applied ater the conversion of the tree.
+      #
+      # Defaults to true.
+      def apply_template_after?
+        true
+      end
+
       # Convert the element tree +tree+ and return the resulting conversion object (normally a
       # string) and an array with warning messages. The parameter +options+ specifies the conversion
       # options that should be used.
       #
       # Initializes a new instance of the calling class and then calls the #convert method with
-      # +tree+ as parameter. If the +template+ option is specified and non-empty, the result is
-      # rendered into the specified template. The template resolution is done in the following way:
+      # +tree+ as parameter.
+      #
+      # If the +template+ option is specified and non-empty, the template is evaluate with ERB
+      # before and/or after the tree conversion depending on the result of #apply_template_before?
+      # and #apply_template_after?. If the template is evaluated before, an empty string is used for
+      # the body; if evaluated after, the result is used as body. See ::apply_template.
+      #
+      # The template resolution is done in the following way (for the converter ConverterName):
       #
       # 1. Look in the current working directory for the template.
       #
-      # 2. Append +.convertername+ (e.g. +.html+) to the template name and look for the resulting
-      #    file in the current working directory.
+      # 2. Append +.converter_name+ (e.g. +.html+) to the template name and look for the resulting
+      #    file in the current working directory (the form +.convertername+ is deprecated).
       #
-      # 3. Append +.convertername+ to the template name and look for it in the kramdown data
-      #    directory.
+      # 3. Append +.converter_name+ to the template name and look for it in the kramdown data
+      #    directory (the form +.convertername+ is deprecated).
+      #
+      # 4. Check if the template name starts with 'string://' and if so, strip this prefix away and
+      #    use the rest as template.
       def self.convert(tree, options = {})
         converter = new(tree, ::Kramdown::Options.merge(options.merge(tree.options[:options] || {})))
+
+        apply_template(converter, '') if !converter.options[:template].empty? && converter.apply_template_before?
         result = converter.convert(tree)
-        result.encode!(tree.options[:encoding]) if result.respond_to?(:encode!)
-        result = apply_template(converter, result) if !converter.options[:template].empty?
+        result.encode!(tree.options[:encoding]) if result.respond_to?(:encode!) && result.encoding != Encoding::BINARY
+        result = apply_template(converter, result) if !converter.options[:template].empty? && converter.apply_template_after?
+
         [result, converter.warnings]
       end
 
@@ -90,6 +117,9 @@ module Kramdown
       end
 
       # Apply the +template+ using +body+ as the body string.
+      #
+      # The template is evaluated using ERB and the body is available in the @body instance variable
+      # and the converter object in the @converter instance variable.
       def self.apply_template(converter, body) # :nodoc:
         erb = ERB.new(get_template(converter.options[:template]))
         obj = Object.new
@@ -99,7 +129,8 @@ module Kramdown
       end
 
       # Return the template specified by +template+.
-      def self.get_template(template) # :nodoc:
+      def self.get_template(template)
+        #DEPRECATED: use content of #get_template_new in 2.0
         format_ext = '.' + self.name.split(/::/).last.downcase
         shipped = File.join(::Kramdown.data_dir, template + format_ext)
         if File.exist?(template)
@@ -108,6 +139,24 @@ module Kramdown
           File.read(template + format_ext)
         elsif File.exist?(shipped)
           File.read(shipped)
+        elsif template.start_with?('string://')
+          template.sub(/\Astring:\/\//, '')
+        else
+          get_template_new(template)
+        end
+      end
+
+      def self.get_template_new(template) # :nodoc:
+        format_ext = '.' + ::Kramdown::Utils.snake_case(self.name.split(/::/).last)
+        shipped = File.join(::Kramdown.data_dir, template + format_ext)
+        if File.exist?(template)
+          File.read(template)
+        elsif File.exist?(template + format_ext)
+          File.read(template + format_ext)
+        elsif File.exist?(shipped)
+          File.read(shipped)
+        elsif template.start_with?('string://')
+          template.sub(/\Astring:\/\//, '')
         else
           raise "The specified template file #{template} does not exist"
         end
@@ -121,7 +170,7 @@ module Kramdown
       # Return +true+ if the header element +el+ should be used for the table of contents (as
       # specified by the +toc_levels+ option).
       def in_toc?(el)
-        @options[:toc_levels].include?(el.options[:level]) and ((not el.attr['class'].split.include?('no_toc')) rescue true)
+        @options[:toc_levels].include?(el.options[:level]) && (el.attr['class'] || '') !~ /\bno_toc\b/
       end
 
       # Return the output header level given a level.
@@ -133,8 +182,8 @@ module Kramdown
 
       # Extract the code block/span language from the attributes.
       def extract_code_language(attr)
-        if attr['class'] && attr['class'] =~ /\blanguage-\w+\b/
-          attr['class'].scan(/\blanguage-(\w+)\b/).first.first
+        if attr['class'] && attr['class'] =~ /\blanguage-\w[\w-]*\b/
+          attr['class'].scan(/\blanguage-(\w[\w-]*)\b/).first.first
         end
       end
 
@@ -143,11 +192,39 @@ module Kramdown
       # *Warning*: This version will modify the given attributes if a language is present.
       def extract_code_language!(attr)
         lang = extract_code_language(attr)
-        attr['class'] = attr['class'].sub(/\blanguage-\w+\b/, '').strip if lang
+        attr['class'] = attr['class'].sub(/\blanguage-\w[\w-]*\b/, '').strip if lang
         attr.delete('class') if lang && attr['class'].empty?
         lang
       end
 
+      # Highlight the given +text+ in the language +lang+ with the syntax highlighter configured
+      # through the option 'syntax_highlighter'.
+      def highlight_code(text, lang, type, opts = {})
+        return nil unless @options[:syntax_highlighter]
+
+        highlighter = ::Kramdown::Converter.syntax_highlighter(@options[:syntax_highlighter])
+        if highlighter
+          highlighter.call(self, text, lang, type, opts)
+        else
+          warning("The configured syntax highlighter #{@options[:syntax_highlighter]} is not available.")
+          nil
+        end
+      end
+
+      # Format the given math element with the math engine configured through the option
+      # 'math_engine'.
+      def format_math(el, opts = {})
+        return nil unless @options[:math_engine]
+
+        engine = ::Kramdown::Converter.math_engine(@options[:math_engine])
+        if engine
+          engine.call(self, el, opts)
+        else
+          warning("The configured math engine #{@options[:math_engine]} is not available.")
+          nil
+        end
+      end
+
       # Generate an unique alpha-numeric ID from the the string +str+ for use as a header ID.
       #
       # Uses the option +auto_id_prefix+: the value of this option is prepended to every generated
