@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #--
-# Copyright (C) 2009-2014 Thomas Leitner <t_leitner@gmx.at>
+# Copyright (C) 2009-2015 Thomas Leitner <t_leitner@gmx.at>
 #
 # This file is part of kramdown which is licensed under the MIT.
 #++
@@ -117,7 +117,7 @@ module Kramdown
           end.each do |k,v|
             warning("Unknown kramdown option '#{k}'")
           end
-          @tree.children << Element.new(:eob, :extension) if type == :block
+          @tree.children << new_block_el(:eob, :extension) if type == :block
           true
         else
           false
@@ -129,7 +129,7 @@ module Kramdown
       ALD_ANY_CHARS = /\\\}|[^\}]/
       ALD_ID_NAME = /\w#{ALD_ID_CHARS}*/
       ALD_TYPE_KEY_VALUE_PAIR = /(#{ALD_ID_NAME})=("|')((?:\\\}|\\\2|[^\}\2])*?)\2/
-      ALD_TYPE_CLASS_NAME = /\.(#{ALD_ID_NAME})/
+      ALD_TYPE_CLASS_NAME = /\.(-?#{ALD_ID_NAME})/
       ALD_TYPE_ID_NAME = /#([A-Za-z][\w:-]*)/
       ALD_TYPE_ID_OR_CLASS = /#{ALD_TYPE_ID_NAME}|#{ALD_TYPE_CLASS_NAME}/
       ALD_TYPE_ID_OR_CLASS_MULTI = /((?:#{ALD_TYPE_ID_NAME}|#{ALD_TYPE_CLASS_NAME})+)/
@@ -152,16 +152,17 @@ module Kramdown
       def parse_block_extensions
         if @src.scan(ALD_START)
           parse_attribute_list(@src[2], @alds[@src[1]] ||= Utils::OrderedHash.new)
-          @tree.children << Element.new(:eob, :ald)
+          @tree.children << new_block_el(:eob, :ald)
           true
         elsif @src.check(EXT_BLOCK_START)
           parse_extension_start_tag(:block)
         elsif @src.scan(IAL_BLOCK_START)
-          if @tree.children.last && @tree.children.last.type != :blank && @tree.children.last.type != :eob
+          if @tree.children.last && @tree.children.last.type != :blank &&
+              (@tree.children.last.type != :eob || [:link_def, :abbrev_def, :footnote_def].include?(@tree.children.last.value))
             parse_attribute_list(@src[1], @tree.children.last.options[:ial] ||= Utils::OrderedHash.new)
-            @tree.children << Element.new(:eob, :ial) unless @src.check(IAL_BLOCK_START)
+            @tree.children << new_block_el(:eob, :ial) unless @src.check(IAL_BLOCK_START)
           else
-            parse_attribute_list(@src[1], @block_ial = Utils::OrderedHash.new)
+            parse_attribute_list(@src[1], @block_ial ||= Utils::OrderedHash.new)
           end
           true
         else
