@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #--
-# Copyright (C) 2009-2013 Thomas Leitner <t_leitner@gmx.at>
+# Copyright (C) 2009-2015 Thomas Leitner <t_leitner@gmx.at>
 #
 # This file is part of kramdown which is licensed under the MIT.
 #++
@@ -18,7 +18,7 @@ module Kramdown
     class Kramdown
 
       LAZY_END_HTML_SPAN_ELEMENTS = HTML_SPAN_ELEMENTS + %w{script}
-      LAZY_END_HTML_START = /<(?>(?!(?:#{LAZY_END_HTML_SPAN_ELEMENTS.join('|')})\b)#{REXML::Parsers::BaseParser::UNAME_STR})\s*(?>\s+#{REXML::Parsers::BaseParser::UNAME_STR}\s*=\s*(["']).*?\1)*\s*\/?>/m
+      LAZY_END_HTML_START = /<(?>(?!(?:#{LAZY_END_HTML_SPAN_ELEMENTS.join('|')})\b)#{REXML::Parsers::BaseParser::UNAME_STR})/
       LAZY_END_HTML_STOP = /<\/(?!(?:#{LAZY_END_HTML_SPAN_ELEMENTS.join('|')})\b)#{REXML::Parsers::BaseParser::UNAME_STR}\s*>/m
 
       LAZY_END = /#{BLANK_LINE}|#{IAL_BLOCK_START}|#{EOB_MARKER}|^#{OPT_SPACE}#{LAZY_END_HTML_STOP}|^#{OPT_SPACE}#{LAZY_END_HTML_START}|\Z/
@@ -29,17 +29,18 @@ module Kramdown
 
       # Parse the paragraph at the current location.
       def parse_paragraph
+        start_line_number = @src.current_line_number
         result = @src.scan(PARAGRAPH_MATCH)
         while !@src.match?(self.class::PARAGRAPH_END)
           result << @src.scan(PARAGRAPH_MATCH)
         end
-        result.chomp!
+        result.rstrip!
         if @tree.children.last && @tree.children.last.type == :p
           @tree.children.last.children.first.value << "\n" << result
         else
-          @tree.children << new_block_el(:p)
+          @tree.children << new_block_el(:p, nil, nil, :location => start_line_number)
           result.lstrip!
-          @tree.children.last.children << Element.new(@text_type, result)
+          add_text(result, @tree.children.last)
         end
         true
       end
